package org.jboss.cache;

import org.jboss.cache.mvcc.ReadCommittedNode;
import org.jboss.cache.optimistic.TransactionWorkspace;
import org.jboss.cache.optimistic.WorkspaceNode;

import java.util.Map;

/**
 * An interface for a factory that creates nodes.  This used to be a concrete class prior to 3.0.0.  Made into an
 * interface to simplify logic of different locking schemes and node types.
 *
 * @author Manik Surtani (<a href="mailto:manik@jboss.org">manik@jboss.org</a>)
 * @since 3.0
 */
public interface NodeFactory<K, V>
{
   ReadCommittedNode createWrappedNode(InternalNode<K, V> node);

   WorkspaceNode<K, V> createWrappedNode(NodeSPI<K, V> dataNode, TransactionWorkspace workspace);

   /**
    * Creates a new node and populates its attributes.
    * <p/>
    * Note that the data map passed in must not be null, and must not be referenced anywhere else as a defensive copy
    * is NOT made when injecting it into the node.
    *
    * @param fqn
    * @param parent
    * @param data
    * @return a new node
    */
   NodeSPI<K, V> createNode(Fqn fqn, NodeSPI<K, V> parent, Map<K, V> data);

   /**
    * Creates a new node and populates its attributes.
    * <p/>
    * Note that the data map passed in must not be null, and must not be referenced anywhere else as a defensive copy
    * is NOT made when injecting it into the node.
    *
    * @param childName
    * @param parent
    * @param data
    * @return a new node
    */
   NodeSPI<K, V> createNode(Object childName, NodeSPI<K, V> parent, Map<K, V> data);

   /**
    * Creates a new, empty node.
    *
    * @param fqn
    * @param parent
    * @return a new node
    */
   NodeSPI<K, V> createNode(Fqn fqn, NodeSPI<K, V> parent);

   /**
    * Creates a new, empty node.
    *
    * @param childName
    * @param parent
    * @return a new node
    */
   NodeSPI<K, V> createNode(Object childName, NodeSPI<K, V> parent);

   /**
    * Creates a new node, and optionally attaches the node to its parent.
    * <p/>
    * The assumption here is that any locks are acquired to prevent concurrent creation of the same node.  Implementations
    * of the NodeFactory should not attempt to synchronize or guard against concurrent creation.
    * <p/>
    *
    * @param fqn            fqn of node to create.  Must not be null or root.
    * @param parent         parent to attach to.  Must not be null, even if attachToParent is false.
    * @param ctx            invocation context to register with.  Must not be null.
    * @param attachToParent if true, the node is registered in the parent's child map.  If false, it is not.
    * @return a new node, or the existing node if one existed.
    */
   InternalNode<K, V> createChildNode(Fqn fqn, InternalNode<K, V> parent, InvocationContext ctx, boolean attachToParent);

   NodeSPI<K, V> createRootNode();

   /**
    * Creates an internal node.  Similar to {@link #createNode(Fqn, NodeSPI)} except that the resultant node is not wrapped
    * in a {@link org.jboss.cache.invocation.NodeInvocationDelegate}.
    *
    * @param childFqn
    * @return a new node
    */
   InternalNode<K, V> createInternalNode(Fqn childFqn);
}
