package org.jboss.cache.batch;

import org.jboss.cache.CacheException;
import org.jboss.cache.factories.annotations.Inject;

import javax.transaction.Transaction;
import javax.transaction.TransactionManager;

/**
 * A container for holding thread locals for batching, to be used with the {@link org.jboss.cache.Cache#startBatch()} and
 * {@link org.jboss.cache.Cache#endBatch(boolean)} calls.
 *
 * @author Manik Surtani (<a href="mailto:manik@jboss.org">manik@jboss.org</a>)
 * @since 3.0
 */
public class BatchContainer
{
   TransactionManager transactionManager;
   private ThreadLocal<Transaction> batchTransactionContainer = new ThreadLocal<Transaction>();

   @Inject
   void inject(TransactionManager transactionManager)
   {
      this.transactionManager = transactionManager;
   }

   public void startBatch() throws CacheException
   {
      try
      {
         if (transactionManager.getTransaction() != null) return;
         if (batchTransactionContainer.get() == null)
         {
            transactionManager.begin();
            batchTransactionContainer.set(transactionManager.suspend());
         }
      }
      catch (Exception e)
      {
         throw new CacheException("Unable to start batch", e);
      }
   }

   public void endBatch(boolean success)
   {
      Transaction tx = batchTransactionContainer.get();
      if (tx == null) return;
      Transaction existingTx = null;
      try
      {
         existingTx = transactionManager.getTransaction();
         transactionManager.resume(tx);
         if (success)
            tx.commit();
         else
            tx.rollback();
      }
      catch (Exception e)
      {
         throw new CacheException("Unable to end batch", e);
      }
      finally
      {
         batchTransactionContainer.remove();
         try
         {
            if (existingTx != null) transactionManager.resume(existingTx);
         }
         catch (Exception e)
         {
            throw new CacheException("Failed resuming existing transaction " + existingTx, e);
         }
      }
   }

   public Transaction getBatchTransaction()
   {
      return batchTransactionContainer.get();
   }
}
