package org.jboss.cache.api.batch;

import org.jboss.cache.Cache;
import org.jboss.cache.CacheFactory;
import org.jboss.cache.DefaultCacheFactory;
import org.jboss.cache.config.Configuration;
import org.jboss.cache.config.Configuration.CacheMode;
import org.jboss.cache.config.Configuration.NodeLockingScheme;
import org.jboss.cache.factories.UnitTestCacheConfigurationFactory;
import org.jboss.cache.util.TestingUtil;
import org.testng.annotations.Test;

import javax.transaction.TransactionManager;

@Test(groups = {"functional", "transaction"})
public class BatchWithTM extends AbstractBatchTest
{
   public void testBatchWithOngoingTM() throws Exception
   {
      Cache<String, String> cache = null;
      try
      {
         cache = createCache();
         TransactionManager tm = getTransactionManager(cache);
         tm.begin();
         cache.put("/a/b/c", "k", "v");
         cache.startBatch();
         cache.put("/a/b/c", "k2", "v2");
         tm.commit();

         assert "v".equals(cache.get("/a/b/c", "k"));
         assert "v2".equals(cache.get("/a/b/c", "k2"));

         cache.endBatch(false); // should be a no op
         assert "v".equals(cache.get("/a/b/c", "k"));
         assert "v2".equals(cache.get("/a/b/c", "k2"));
      }
      finally
      {
         TestingUtil.killCaches(cache);
      }
   }

   public void testBatchWithoutOngoingTMSuspension() throws Exception
   {
      Cache<String, String> cache = null;
      try
      {
         cache = createCache();
         TransactionManager tm = getTransactionManager(cache);
         assert tm.getTransaction() == null : "Should have no ongoing txs";
         cache.startBatch();
         cache.put("/a/b/c", "k", "v");
         assert tm.getTransaction() == null : "Should have no ongoing txs";
         cache.put("/a/b/c", "k2", "v2");

         assert getOnDifferentThread(cache, "/a/b/c", "k") == null;
         assert getOnDifferentThread(cache, "/a/b/c", "k2") == null;

         try
         {
            tm.commit(); // should have no effect
         }
         catch (Exception e)
         {
            // the TM may barf here ... this is OK.
         }

         assert tm.getTransaction() == null : "Should have no ongoing txs";

         assert getOnDifferentThread(cache, "/a/b/c", "k") == null;
         assert getOnDifferentThread(cache, "/a/b/c", "k2") == null;

         cache.endBatch(true); // should be a no op

         assert "v".equals(getOnDifferentThread(cache, "/a/b/c", "k"));
         assert "v2".equals(getOnDifferentThread(cache, "/a/b/c", "k2"));
      }
      finally
      {
         TestingUtil.killCaches(cache);
      }
   }

   public void testBatchRollback() throws Exception
   {
      Cache<String, String> cache = null;
      try
      {
         cache = createCache();
         TransactionManager tm = getTransactionManager(cache);
         cache.startBatch();
         cache.put("/a/b/c", "k", "v");
         cache.put("/a/b/c", "k2", "v2");

         assert getOnDifferentThread(cache, "/a/b/c", "k") == null;
         assert getOnDifferentThread(cache, "/a/b/c", "k2") == null;

         cache.endBatch(false);

         assert getOnDifferentThread(cache, "/a/b/c", "k") == null;
         assert getOnDifferentThread(cache, "/a/b/c", "k2") == null;
      }
      finally
      {
         TestingUtil.killCaches(cache);
      }
   }

   private TransactionManager getTransactionManager(Cache<String, String> c)
   {
      return c.getConfiguration().getRuntimeConfig().getTransactionManager();
   }

   private Cache<String, String> createCache()
   {
      CacheFactory<String, String> cf = new DefaultCacheFactory<String, String>();
      Configuration c = UnitTestCacheConfigurationFactory.createConfiguration(CacheMode.LOCAL); // this should pick up any configured TM for the test
      c.setNodeLockingScheme(NodeLockingScheme.MVCC);
      c.setInvocationBatchingEnabled(true);
      assert c.getTransactionManagerLookupClass() != null : "Should have a transaction manager lookup class attached!!";
      return cf.createCache(c);
   }
}
